
#include <gdt.h>

/* Defines a GDT entry */
struct gdt_entry {
	// unsigned short limit_low;
	uint16_t limit_low; 
	//unsigned short base_low;
	uint16_t base_low;
	//unsigned char base_middle;
	uint8_t base_middle;
	//unsigned char access;
	uint8_t access;
	//unsigned char granularity;
	uint8_t granularity;
	//unsigned char base_high;
	uint8_t base_high;
}__attribute__((packed));

struct gdt_ptr {
	unsigned short limit;
	//unsigned int base;
	void *base;
}__attribute__((packed));

struct gdt_entry gdt[3];
struct gdt_ptr _gdt_p;

extern void _gdt_flush();

void gdt_set_gate(int num, uint32_t base, uint32_t limit,
		uint8_t access, uint8_t gran) {
	/* Setup the descriptor base address */
	gdt[num].base_low = (base & 0xFFFF);
	gdt[num].base_middle = (base >> 16) & 0xFF;
	gdt[num].base_high = (base >> 24) & 0xFF;

	/* Setup the descriptor limits */
	gdt[num].limit_low = (limit & 0xFFFF);
	gdt[num].granularity = ((limit >> 16) & 0x0F);

	/* Finally, set up the granularity and access flags */
	gdt[num].granularity |= (gran & 0xF0);
	gdt[num].access = access;
}

void gdt_install() {
	/* Setup the GDT pointer and limit */
	_gdt_p.limit = (sizeof(struct gdt_entry) * 3) - 1;
	//_gdt_p.base = (unsigned int)(&gdt);
	_gdt_p.base = &gdt;
	
	/* NULL descriptor */
	gdt_set_gate(0, 0, 0, 0, 0);

	/*  Code Segment. */
	gdt_set_gate(1, 0, 0xFFFFFFFF, 0x9A, 0xCF);

	/*  Data Segment.*/
	gdt_set_gate(2, 0, 0xFFFFFFFF, 0x92, 0xCF);

	_gdt_flush();
}
