
#include <idt.h>
#include <libc.h>

/* Defines an IDT entry */
struct idt_entry {
	//unsigned short base_lo;
	uint16_t base_lo;
	//unsigned short sel;
	uint16_t sel;
	//unsigned char always0;
	uint8_t always0;
	//unsigned char flags;
	uint8_t flags;
	//unsigned short base_hi;
	uint8_t base_hi;
}__attribute__((packed));

struct idt_ptr {
	//unsigned short limit;
	uint16_t limit;
	//unsigned int base;
	void *base;
}__attribute__((packed));

/*  If any undefined IDT entry is hit, it normally
 *  will cause an "Unhandled Interrupt" exception. Any descriptor
 *  for which the 'presence' bit is cleared (0) will generate an
 *  "Unhandled Interrupt" exception */
struct idt_entry idt[256];
struct idt_ptr _idt_p;


extern void _idt_load();

/* Use this function to set an entry in the IDT. Alot simpler
 *  than twiddling with the GDT ;) */
void idt_set_gate(uint8_t num, uint32_t base, uint16_t sel,
		uint8_t flags) {
	/* The interrupt routine's base address */
	idt[num].base_lo = (base & 0xFFFF);
	idt[num].base_hi = (base >> 16) & 0xFFFF;

	/* The segment or 'selector' that this IDT entry will use
	 *  is set here, along with any access flags */
	idt[num].sel = sel;
	idt[num].always0 = 0;
	idt[num].flags = flags;
}

void idt_install() {
	_idt_p.limit = (sizeof(struct idt_entry) * 256) - 1;
	_idt_p.base = &idt;

	memset(&idt, 0, sizeof(struct idt_entry) * 256);

	/* Add any new ISRs to the IDT here using idt_set_gate */

	/* Points the processor's internal register to the new IDT */
	_idt_load();
}
