
#include <io.h>

void outb(uint16_t port, uint8_t byte) {
	asm("mov %0, %%dx\n"
		"mov %1, %%al\n"
		"outb %%al, %%dx\n"
	    :  : "g" (port), "g" (byte)
		: "dx","ax"
	);
}


void outw(uint16_t port, uint16_t word) {
	asm("\n"
		"mov %0, %%dx\n"
		"mov %1, %%ax\n"
		"outw %%ax, %%dx\n"
		: : "g" (port), "g" (word)
		: "dx","ax"
	);
}

void outl(uint16_t port, uint32_t l) {
	asm("\n"
		"mov %0, %%dx\n"
		"mov %1, %%eax\n"
		"outl %%eax, %%dx\n"
		: : "g" (port), "g" (l)
		: "dx","eax"
	);
}


uint32_t inl(uint16_t port) {
	uint32_t ret = 0;
	asm("\n"
		"mov %1, %%dx\n"
		"inl %%dx, %%eax\n"
		"mov %%eax, %0\n"
		: "=g" (ret) : "g" (port)
		: "dx","eax"
	);
	return ret;
}



uint16_t inw(uint16_t port) {
	uint16_t ret = 0;
	asm("\n"
		"mov %1, %%dx\n"
		"inw %%dx, %%ax\n"
		"mov %%ax, %0\n"
		: "=g" (ret) : "g" (port)
		: "dx","ax"
	);
	return ret;
}

uint8_t inb(uint16_t port) {
	char ret = 0;
	asm("\n"
		"mov %1, %%dx\n"
		"inb %%dx, %%al\n"
		"mov %%al, %0\n"
		: "=g" (ret) : "g" (port)
		: "dx","ax"
	);
	return ret;
}

void outb_pulse(uint16_t port,  uint8_t byte) {
	outb(port, byte);
	SLOW_DOWN_IO;
}

void outw_pulse(uint16_t port, uint16_t word) {
	outw(port, word);
	SLOW_DOWN_IO
;}

void outl_pulse(uint16_t port, uint32_t word) {
	outl(port, word);
	SLOW_DOWN_IO
;}

uint32_t inl_pulse(uint16_t port) {
	uint32_t ret = 0;
	ret = inl(port);
	SLOW_DOWN_IO
;	return ret;
}


uint16_t inw_pulse(uint16_t port) {
	uint16_t ret = 0;
	ret = inw(port);
	SLOW_DOWN_IO
;	return ret;
}

uint8_t inb_pulse(uint16_t port) {
	char ret = 0;
	ret = inb(port);
	SLOW_DOWN_IO
;	return ret;
}



/* old definitions
 * sizeof(unsigned short) = 2 bytes
 * 
inline unsigned char inportb (unsigned short _port)
{
    unsigned char rv;
    __asm__ __volatile__ ("inb %1, %0" : "=a" (rv) : "dN" (_port));
    return rv;
}

inline void outportb (unsigned short _port, unsigned char _data)
{
    __asm__ __volatile__ ("outb %1, %0" : : "dN" (_port), "a" (_data));
}

*/
